/**
 **
 **   LJ -- A printing utility for the HP LaserJet
 **
 **
 **   This program prints a series of files on the LaserJet printer.  The
 **   files are printed in a "landscape" font at 17 characters to the inch.
 **   To take advantage of this density, two "pages" of information from
 **   the file are printed on each piece of paper (left and right halves).
 **
 **   Usage is:       LJ  file1 file2 file3 ...
 **
 **   Where file# is a valid MS-DOS filename, included on the command line.
 **
 **   Originally written by Joe Barnhart and subsequently modifed by:
 **     Ray Duncan and Chip Rabinowitz.  This program has been placed in
 **     the public domain for use without profit.
 **
 **   Revised 9/86 for the Mark Williams C Programming System,
 **     Steven Stern, JMB Realty Corp.
 **
 **   Revised 11/86 for DOS wild card characters in the file name.
 ****************************************************************************
 **
 **   Revised 4/88 for the DATALIGHT C Compiler by Serge Stepanoff,
 **   STS Enterprises, 5469 Arlene Way, Livermore, CA.
 **
 **	The revised code uses the built in file wildcard expansion feature
 **	of DATALIGHT C, and makes use of some additional flags to
 **	turn off the Header line, optionally use the 7 point
 **	Prestige Elite font, and ssome oher features.  Typing "lj2" with
 **	no parameters will produce a short users' explanaion.
 **
 **	Program has been checked out on an OKIDATA
 **	LASERLINE printer (HP Laserjet compatible).
 *****************************************************************************
 ** Revised 10/16/89 by Jim Derr for the Turbo C 2.0 Compiler.
 ** Uses the wildargs obj module for file wild card expansion.
 ** Corrected some anomalies in the code that was causing invalid pages breaks.
 ** Added the -s command line option that will shade every other line on the
 ** printout for readability.
 *****************************************************************************
 **
 ** Revised 1/20/90 by Blake C. Ramsdell to work correctly on the LaserJet IIP
 ** (/2 compatibility switch).  Also corrected problems in "switch" statements
 ** regarding duplicate cases (EOF is the same as \377 octal (255 decimal) in
 ** Turbo C.  If you define the constant "LJ2P" below, the /2 option will
 ** make correct output for the LaserJet II.  Basically, there is a small
 ** discrepancy in the LaserJet IIp in the page size.  I just changed the
 ** top margin command ("\033&l6E" for the IIp, "\033&l7E" for the II) to make
 ** things OK.
 **/

#include "stdio.h"
#include "time.h"
#include "ctype.h"
#include <fcntl.h>

#define MAXLINE  66         /* maximum lines per page in this program */

/* #define LJ2P */	     /* to print to a IIp by default, define this */
#define lp   stdprn

/* FILE *lp; */

FILE *fp;               /* FILE pointer for the file to be printed */
int  hdrflag;		/* Header print suppress flag */
int  pgflag;		/* Page separator flag */
int  tab = 4;		/* default value of one tab stop */
int  pgside;		/* left/right half of page indicator  */
int  psprint = 0;   /* proportional using Helv 6 pt. */
int  shade_it = 0;  /* shade every other line */
int  maxcol=80;

/*************************************************************************

main

*************************************************************************/

main(argc, argv)
int argc;
char *argv[];
{
int filenum;
char fname[70];
int first;
int jim;
FILE *mine;

   if (argc <= 1)
      {
      usage();
      exit(1);
      }


   _fmode = O_TEXT;
	mine = freopen("PRN","w",stdprn);

/*  lp = fopen("lj2.out","wt");  */

/* initialize the LaserJet for landscape printing */

/*   fprintf( lp,"\033E\033&l1O\033(s17H\033&l5.14C\033&l70F\033&l7E" ); */
   fprintf( lp,"\033E\033&l1O\033(s17H\033&l5.14C\033&l70F" );

#if defined (LJ2P)
   fprintf(lp,"\033&l6E");
#else
   fprintf(lp,"\033&l7E");
#endif

   for (first = 1; first < argc; first++)  {
      if (*argv[first] != '/' && *argv[first] != '-') /* check for flags */
         break;					/* and exit if none */
      argv[first]++;
      switch (tolower(*argv[first]))  {
         case 'e':		/* Prestige Elite 7 point font */
            psprint = 1;
            maxcol=100;
            fprintf(lp,"\033&l1O\033(0U\033(s1p6vsb4T");
            break;
         case 'h':		/* suppress headers flag */
            hdrflag++;
            break;
         case 't':		/* Horizontal tab value */
            tab = atoi(++argv[first]);
            if (tab < 1 || tab > 8)  {
                printf("Invalid tab value specified -- defaulting to 8\n");
                tab = 8;
                }
			break;
		 case 'p':		/* suppress page eject between files */
			pgflag++;
			break;
		 case 's':
			shade_it++;
			break;
	 case '2':
#if defined (LJ2P)
            /* Set the correct top margin for the LaserJet II */
	    fprintf(lp,"\033&l7E");
#else
	    /* Set the correct top margin for the LaserJet IIp */
	    fprintf(lp,"\033&l6E");
#endif
	    break;
         default:
            printf("Invalid Flag Specified ('%s') -- ignored.....\n",
            		--argv[first]);
            break;
         }
      }

   for(filenum = first; filenum < argc; filenum++ )
      {
         copyname(fname, argv[filenum]);
         fp = fopen(fname ,"r");
         if( fp == NULL )
            {
            printf( "File %s doesn't exist.\n", fname );
            }
         else
            {
            printf( "Now printing %s\n", fname );
            printfile( fname );
            fclose( fp );
            }
      }    /* of loop through run-time args */

/*
   if (pgflag & pgside)
	fputc('\f', lp);
*/
   fprintf( lp, "\r\033E" );      /* clear LaserJet */
   fclose(lp);


}

/*************************************************************************

printfile (filename)

*************************************************************************/

printfile(filename)
char *filename;
{
   int pagenum = 1;
   int retval = 0;
   int printed = 0;
   char c2;

   while( (feof(fp)==0) && (retval==0) )
      {
	  if (pgside == 0)  {
		 printed = 0;
		 c2 = fgetc(fp);
		 switch (c2) {
			case EOF:
/*			case '\377': */
			case '\032':
				retval = -1;
				break;
			default:
				printed=1;
				dovert();
                if(psprint)
                   fprintf(lp,"\033&a0r0c120m5L\r");
                else
                   fprintf(lp, "\033&a0r0c85m5L\r"); /* set LaserJet to left half */
				if(shade_it) shade();
				ungetc(c2,fp);
				header(filename, pagenum++);          /* title top of page */
				retval = printpage();                 /* print one page */
				pgside ^= 1;
		 }
	  }
      if(feof(fp)==0 && retval==0)
         {                                  /* if more to print... */
         if(psprint)
			fprintf(lp,"\033&a0r0c239m124L\r");
         else
			fprintf(lp, "\033&a0r0c171m91L\r");    /* LaserJet to right half */
		 c2 = fgetc(fp);
		 switch (c2) {
			case EOF:
/*			case '\377': */
			case '\032':
				retval = -1;
				break;
			default:
				ungetc(c2,fp);
				header(filename, pagenum++);          /* title top of page */
				retval = printpage();                 /* print one page */
				pgside ^= 1;
		 }
	  }
	  if (pgside == 0 && printed)
         fputc('\f', lp);                          /* kick out paper */
	  else if (pgflag == 0 && printed)  {
      	 fputc('\f', lp);
      	 pgside = 0;
      	 }
      }
   return(0);
}

/*************************************************************************

printpage
   print a logical page

*************************************************************************/

printpage()
{
   char c;
   int line,col;
   static int cont = 0;
   static char *cont1 = "---->";

   line = col = 0;
   if(cont)
      {
      fprintf(lp,cont1);
      col = strlen(cont1);
      cont = 0;
      }

   while( line < MAXLINE )
      {
      c = fgetc(fp);
      if(col>maxcol)
         {
         line++;
         switch(c)
            {
            case '\n':
            case '\r':
            case '\f':
            case EOF:
/*          case '\377': */
            case '\032':
               break;
            default:
               if(line >= MAXLINE)
                  {
                  cont = 1;
                  ungetc(c,fp);
                  return(0);
                  }
               fprintf(lp,"\n%s",cont1);
               col = strlen(cont1);
               break;
            }
         }
      switch(c)
         {
         case '\n':           /* newline found */
            col = 0;          /* zero column and */
            line++;           /* advance line count */
			if( line < MAXLINE )
               fprintf(lp,"\n");
            break;
         case '\r':           /* CR found */
            break;            /* discard it */
         case '\t':           /* TAB found */
            do
               fputc(' ',lp);
            while ( (++col % tab) != 0 );
            break;
		 case '\f':              /* Page break or */
			if(line != 0)
				line = MAXLINE;      /* force termination of loop */
            break;
         case EOF:            /* EOF mark */
/*       case '\377': */      /* EOF mark */ 
         case '\032':         /* EOF mark */
            return(-1);
         default:              /* no special case */
            fputc(c,lp);       /* print character */
            col++;
            break;
      }
   }
   return(0);
}

/*************************************************************************

header
   print a page header

*************************************************************************/

header( filename, pagenum )
char *filename;
int pagenum;
{
   char datestr[11], timestr[11];
   if (hdrflag)  {
      fputc('\n', lp);
      return;		/* skip if flag set */
      }
   timestamp(timestr);
   datestamp(datestr);
   fprintf(lp,"\033&d0D");
   fprintf(lp, "File: %-40s%s   %s  --  Page: %03d\n\n",
	  filename,datestr,timestr,pagenum);
   fprintf(lp,"\033&d@");
}

timestamp( timestr )
char   *timestr;
{
   struct tm *tod;
   time_t tt;
   tt = time(NULL);
   tod = localtime(&tt);
   sprintf(timestr,"%02d:%02d", tod->tm_hour, tod->tm_min);
   return;
}

datestamp( datestr )
char  *datestr;
{
   struct tm *tod;
   time_t tt;
   tt = time(NULL);
   tod = localtime(&tt);
   sprintf(datestr,"%02d/%02d/%02d", tod->tm_mon + 1, tod->tm_mday,
	tod->tm_year + 1900);
   return;
}

/*************************************************************************

dovert()
   draw a vertical line down the center of the physical page

*************************************************************************/

dovert()
{
   int line = 1;

   if(psprint)
		fprintf(lp,"\033&a0r0c123m121L\r|");
   else
		fprintf(lp,"\033&a0r0c90m88L\r|");

   while(line++ < MAXLINE+2) fprintf(lp,"\n|");
}

/*************************************************************************

copyname
	copy a file name converting to upper case.

*************************************************************************/

copyname(to, from)
char *to, *from;
{
	while (*from)
		*to++ = toupper(*from++);
	*to = 0;
}

/************************************************************************/

usage()
{
	printf("\nUSAGE:\n\n");
	printf("C>LJ2 [flags] filename1 [filename2 ..........]\n");
	printf("\tItems shown in brackets are optional.");
	printf("\n\n\tFilename(s) may include wildcards");
	printf("\n\n\tFlags are:\n");
    printf("\t\t-e (or /e) select Helv proportional 6 PT font\n");
	printf("\t\t-h (or /h) suppress page headers\n");
	printf("\t\t-tx (or /tx) set tab stop value (where x is 1 to 8)\n");
	printf("\t\t-p (or /p) do not leave blank half pages between files\n");
	printf("\t\t-s (or /s) shade every other line\n");
#if defined(LJ2P)
	printf("\t\t-2 (or /2) print on LaserJet II instead of IIp\n");
#else
	printf("\t\t-2 (or /2) print on LaserJet IIp instead of II\n");
#endif
}


shade()
{


	int i;
    fprintf (lp, "\033&f0S\033*p0x0Y\033*c3300a32b10G");
	for (i=1; i <= 34; i++)
        fprintf (lp, "\033*c2P\033&a+2R");
    fprintf (lp, "\033&f1S");

}


